# VM creation tutorial

Based on https://github.com/systems-cs-pub-ro/so/tree/master/infrastructure/vm
(which is in turn based on https://gitlab.com/adrian.stratulat/vm-scripts/).

## Setting up the environment (for a Linux x86 building host)


    $ sudo apt-get install vagrant virtualbox virtualbox-qt python-pip python-setuptools ansible
    $ pip install ansible

## Setting up the environment (for a MacOS building host)

In order to `vagrant up` any VM using VMware Fusion, in addition to [VMware Fusion](https://blogs.vmware.com/teamfusion/2024/05/fusion-pro-now-available-free-for-personal-use.html), you will need [Vagrant](https://developer.hashicorp.com/vagrant/install/vmware), vagrant-vmware-utility, and [vagrant-vmware-desktop](https://github.com/hashicorp/vagrant-vmware-desktop) plugin. Ansible should also be installed on the host machine to run our playbooks.

```sh
brew install vagrant ansible
brew install --cask vagrant-vmware-utility
vagrant plugin install vagrant-vmware-desktop
```

## Building a VM

You just need to run this:

```
vagrant up
```

## Exporting the VM

Use the `./export.sh` script to get an `.ova` image.

## Building on top of the SO VM

If you already have the [SO virtual
machine](https://github.com/systems-cs-pub-ro/so/tree/master/infrastructure/vm),
you can skip installing an entirely new VM and just provision the SO one with
the extra packages needed for PCom.

Run:

```
vagrant ssh-config > ssh-config
ansible-playbook --inventory .vagrant/provisioners/ansible/inventory/vagrant_ansible_inventory playbooks/pcom/main.yml --ssh-extra-args="-F ssh-config"
```

If you did not build the SO VM using Vagrant, but simply imported it, then create the file `ssh-config` manually (replace `<path-to-private-key>` with the path of a private key that allows you to SSH into the `vagrant` user on that machine.

```
Host default
  HostName 127.0.0.1
  User vagrant
  Port 2222
  UserKnownHostsFile /dev/null
  StrictHostKeyChecking no
  PasswordAuthentication no
  IdentityFile <path-to-private-key>
  IdentitiesOnly yes
  LogLevel FATAL
  PubkeyAcceptedKeyTypes +ssh-rsa
  HostKeyAlgorithms +ssh-rsa
```

Then create the inventory `vagrant_ansible_inventory` (replacing `<path-to-private-key>` as above):

```
default ansible_ssh_host=127.0.0.1 ansible_ssh_port=2222 ansible_ssh_user='vagrant' ansible_ssh_private_key_file='<path-to-private-key>'
```


## Building on top of another VM

You should, in theory, be able to provision any other similar VM, by making the proper adjustements (i.e. change the user from `vagrant` or the SSH forwarding port from `2222`).
However, there are many limitations; for example, it will only work on Debian-based distros.
